@@ -7,15 +7,15 @@ use std::{
 };
 
 use crate::{
+    FontSize, ImageSource, Resize, Result,
     errors::Errors,
     protocol::{
+        Protocol, StatefulProtocol, StatefulProtocolType,
         halfblocks::Halfblocks,
         iterm2::Iterm2,
         kitty::{Kitty, StatefulKitty},
         sixel::Sixel,
-        Protocol, StatefulProtocol, StatefulProtocolType,
     },
-    FontSize, ImageSource, Resize, Result,
 };
 use cap_parser::{Parser, QueryStdioOptions, Response};
 use image::{DynamicImage, Rgba};
@@ -107,6 +107,17 @@ impl Picker {
         // Detect tmux, and only if positive then take some risky guess for iTerm2 support.
         let (is_tmux, tmux_proto) = detect_tmux_and_outer_protocol_from_env();
 
+        static DEFAULT_PICKER: Picker = Picker {
+            // This is completely arbitrary. For halfblocks, it doesn't have to be precise
+            // since we're not rendering pixels. It should be roughly 1:2 ratio, and some
+            // reasonable size.
+            font_size: (10, 20),
+            background_color: DEFAULT_BACKGROUND,
+            protocol_type: ProtocolType::Halfblocks,
+            is_tmux: false,
+            capabilities: Vec::new(),
+        };
+
         // Write and read to stdin to query protocol capabilities and font-size.
         match query_with_timeout(is_tmux, Duration::from_secs(1), options) {
             Ok((capability_proto, font_size, caps)) => {
@@ -127,19 +138,16 @@ impl Picker {
                         capabilities: caps,
                     })
                 } else {
-                    Err(Errors::NoFontSize)
+                    let mut p = DEFAULT_PICKER.clone();
+                    p.is_tmux = is_tmux;
+                    Ok(p)
                 }
             }
-            Err(Errors::NoCap) => Ok(Self {
-                // This is completely arbitrary. For halfblocks, it doesn't have to be precise
-                // since we're not rendering pixels. It should be roughly 1:2 ratio, and some
-                // reasonable size.
-                font_size: (10, 20),
-                background_color: DEFAULT_BACKGROUND,
-                protocol_type: ProtocolType::Halfblocks,
-                is_tmux,
-                capabilities: Vec::new(),
-            }),
+            Err(Errors::NoCap | Errors::NoStdinResponse | Errors::NoFontSize) => {
+                let mut p = DEFAULT_PICKER.clone();
+                p.is_tmux = is_tmux;
+                Ok(p)
+            }
             Err(err) => Err(err),
         }
     }
@@ -294,6 +302,8 @@ fn iterm2_from_env() -> Option<ProtocolType> {
             || term_program.contains("Tabby")
             || term_program.contains("Hyper")
             || term_program.contains("rio")
+            || term_program.contains("Bobcat")
+            || term_program.contains("WarpTerminal")
     }) {
         return Some(ProtocolType::Iterm2);
     }
@@ -328,7 +338,6 @@ fn enable_raw_mode() -> Result<impl FnOnce() -> Result<()>> {
 #[cfg(windows)]
 fn enable_raw_mode() -> Result<impl FnOnce() -> Result<()>> {
     use windows::{
-        core::PCWSTR,
         Win32::{
             Foundation::{GENERIC_READ, GENERIC_WRITE, HANDLE},
             Storage::FileSystem::{
@@ -338,6 +347,7 @@ fn enable_raw_mode() -> Result<impl FnOnce() -> Result<()>> {
                 self, CONSOLE_MODE, ENABLE_ECHO_INPUT, ENABLE_LINE_INPUT, ENABLE_PROCESSED_INPUT,
             },
         },
+        core::PCWSTR,
     };
 
     let utf16: Vec<u16> = "CONIN$\0".encode_utf16().collect();
@@ -416,21 +426,14 @@ fn query_stdio_capabilities(
     let mut responses = vec![];
     'out: loop {
         let mut charbuf: [u8; 50] = [0; 50];
-        let result = io::stdin().read(&mut charbuf);
-        match result {
-            Ok(read) => {
-                for ch in charbuf.iter().take(read) {
-                    let mut more_caps = parser.push(char::from(*ch));
-                    match more_caps[..] {
-                        [Response::Status] => {
-                            break 'out;
-                        }
-                        _ => responses.append(&mut more_caps),
-                    }
+        let read = io::stdin().read(&mut charbuf)?;
+        for ch in charbuf.iter().take(read) {
+            let mut more_caps = parser.push(char::from(*ch));
+            match more_caps[..] {
+                [Response::Status] => {
+                    break 'out;
                 }
-            }
-            Err(err) => {
-                return Err(err.into());
+                _ => responses.append(&mut more_caps),
             }
         }
     }
@@ -517,16 +520,12 @@ fn query_with_timeout(
     let (tx, rx) = mpsc::channel();
 
     thread::spawn(move || {
-        let _ = tx.send(
-            enable_raw_mode()
-                .map_err(Errors::into)
-                .and_then(|disable_raw_mode| {
-                    let result = query_stdio_capabilities(is_tmux, options);
-                    // Always try to return to raw_mode.
-                    disable_raw_mode()?;
-                    result
-                }),
-        );
+        let _ = tx.send(enable_raw_mode().and_then(|disable_raw_mode| {
+            let result = query_stdio_capabilities(is_tmux, options);
+            // Always try to return to raw_mode.
+            disable_raw_mode()?;
+            result
+        }));
     });
 
     match rx.recv_timeout(timeout) {
